@@ -1,15 +1,17 @@
 use crate::menu::{self, Item, Status};
 use chrono::{DateTime, Utc};
+use itertools::{GroupBy, Itertools};
 use serde::{Deserialize, Serialize};
-use std::collections::{BTreeMap, BTreeSet};
+use std::collections::{btree_set, BTreeMap, BTreeSet};
 use std::default::Default;
-use std::io::{self, ErrorKind};
+use std::fs::{read_to_string, File};
+use std::io::{self, ErrorKind, Write};
 use std::mem::swap;
 use std::path::{Path, PathBuf};
-use tokio::fs::{read_to_string, File};
-use tokio::io::AsyncWriteExt;
 
 /// `Exercise` represents an exercise.
+///
+/// TODO Fix implicit equality and ordering of exercises
 #[derive(Debug, Clone)]
 pub struct Exercise {
     pub path: PathBuf,
@@ -131,9 +133,9 @@ impl State {
     ///
     /// Returns an [`io::Error`] if reading the state file fails or if
     /// the file contains invalid toml.
-    pub async fn open(path: impl AsRef<Path>) -> io::Result<Self> {
+    pub fn open(path: impl AsRef<Path>) -> io::Result<Self> {
         let map: BTreeMap<PathBuf, ExerciseState> =
-            toml::from_str(&read_to_string(path.as_ref().join("state.toml")).await?)?;
+            toml::from_str(&read_to_string(path.as_ref().join("state.toml"))?)?;
         Ok(map.into())
     }
 
@@ -144,9 +146,9 @@ impl State {
     /// # Errors
     ///
     /// Same errors as [`Self::open`].
-    pub async fn scan_open_default(path: impl AsRef<Path>) -> io::Result<Self> {
+    pub fn scan_open_default(path: impl AsRef<Path>) -> io::Result<Self> {
         let path = path.as_ref();
-        let mut state = match Self::open(path).await {
+        let mut state = match Self::open(path) {
             Err(e) => {
                 if e.kind() == ErrorKind::NotFound {
                     Self::default()
@@ -167,8 +169,8 @@ impl State {
     ///
     /// Returns an [`io::Error`] if creating or writing to the state
     /// file fails.
-    pub async fn save(&self) -> io::Result<()> {
-        let mut file = File::create("exercises/state.toml").await?;
+    pub fn save(&self) -> io::Result<()> {
+        let mut file = File::create("exercises/state.toml")?;
         let mut map = BTreeMap::new();
 
         for ex in &self.0 {
@@ -178,8 +180,7 @@ impl State {
             toml::to_string(&map)
                 .expect("state to be serializable")
                 .as_bytes(),
-        )
-        .await?;
+        )?;
         Ok(())
     }
 
@@ -238,22 +239,72 @@ impl State {
     pub fn extend(&mut self, mut other: Self) {
         swap(&mut self.0, &mut other.0); // swapping sets inverts the priority
         self.0.append(&mut other.0);
-        // let mut extended = BinaryHeap::with_capacity(self.0.capacity() + other.0.capacity());
-        // while let Some(self_next) = self.0.pop() {
-        //     while let Some(other_next) = other.0.peek() {
-        //         if other_next > &self_next {
-        //             extended.push(other.0.pop().expect("peek and pop to give same value"));
-        //         } else {
-        //             break;
-        //         }
-        //     }
-        //     extended.push(self_next);
-        // }
-        // Self(extended)
     }
 
     /// Returns an iterator yielding [`Exercise`]s.
     pub fn iter(&self) -> impl Iterator<Item = &'_ Exercise> {
         self.0.iter()
     }
+
+    /// Returns an iterator yielding a tuple of folder and the exercises within.
+    pub fn group_by_folder<'s>(
+        &'s self,
+    ) -> GroupBy<&'_ str, btree_set::Iter<'_, Exercise>, for<'r> fn(&'r &'s Exercise) -> &'s str>
+    {
+        self.0.iter().group_by(|t| t.folder_name())
+    }
+
+    /// Returns the next exercise of the week or `None` if the exercise is the last one in the folder.
+    ///
+    /// # Examples
+    ///
+    /// Get the next exercise `ex2` when `ex1` comes before `ex2` in `week1`:
+    ///
+    /// ```
+    /// # use chrono::Utc;
+    /// # use otarustlings::state::{State, Exercise, ExerciseState};
+    /// let started = ExerciseState {
+    ///     started: Some(Utc::now()),
+    ///     ..ExerciseState::default()
+    /// };
+    ///
+    /// let ex1 = Exercise { path: "week1/ex1.rs".into(), exercise_state: started.clone() };
+    /// let ex2 = Exercise { path: "week1/ex2.rs".into(), exercise_state: ExerciseState::default() };
+    /// let mut state = State::new();
+    /// state.0.insert(ex1.clone());
+    /// state.0.insert(ex2.clone());
+    ///
+    /// assert_eq!(state.get_next_exercise(&ex1), Some(&ex2));
+    /// ```
+    ///
+    /// Should return `None` if the exercise is last exercise of the week.
+    ///
+    /// ```
+    /// # use chrono::Utc;
+    /// # use otarustlings::state::{State, Exercise, ExerciseState};
+    /// let started = ExerciseState {
+    ///     started: Some(Utc::now()),
+    ///     ..ExerciseState::default()
+    /// };
+    ///
+    /// let w1ex1 = Exercise { path: "week1/ex1.rs".into(), exercise_state: started.clone() };
+    /// let w2ex1 = Exercise { path: "week2/ex1.rs".into(), exercise_state: ExerciseState::default() };
+    /// let mut state = State::new();
+    /// state.0.insert(w1ex1.clone());
+    /// state.0.insert(w2ex1.clone());
+    ///
+    /// assert_eq!(state.get_next_exercise(&w1ex1), None);
+    /// ```
+    pub fn get_next_exercise(&self, exercise: &Exercise) -> Option<&Exercise> {
+        let mut exercises = self.0.iter();
+        exercises.find(|&x| x == exercise);
+        // Return None if the next exercise does not exist
+        let not_last_exercise = exercises.next()?;
+        // Return None if the next exercise is not in the same folder
+        if not_last_exercise.folder_name() != exercise.folder_name() {
+            None
+        } else {
+            Some(not_last_exercise)
+        }
+    }
 }
